/*  Sol's 2d gl basecode
 *  version 2.0, February 19th, 2008
 *
 *  Copyright (C) 2008 Jari Komppa
 *
 *  This software is provided 'as-is', without any express or implied
 *  warranty.  In no event will the authors be held liable for any damages
 *  arising from the use of this software.
 *
 *  Permission is granted to anyone to use this software for any purpose,
 *  including commercial applications, and to alter it and redistribute it
 *  freely, subject to the following restrictions:
 *
 *  1. The origin of this software must not be misrepresented; you must not
 *     claim that you wrote the original software. If you use this software
 *     in a product, an acknowledgment in the product documentation would be
 *     appreciated but is not required.
 *  2. Altered source versions must be plainly marked as such, and must not be
 *     misrepresented as being the original software.
 *  3. This notice may not be removed or altered from any source distribution.
 *
 * Jari Komppa http://iki.fi/sol/
 *
 *************************************
 * 
 */

#include "toolkit.h"

#define TITLE "Wall Defence 1.0"

// How big a tick difference is considered 'time warp', i.e. skip the time
// (to avoid physics blowing up)
#define TICK_TIMEWARP 1000
// Physics ticks per second
#define PHYSICS_FPS 100
// For windowed mode
#define DESIRED_WINDOW_WIDTH 800
// For windowed mode
#define DESIRED_WINDOW_HEIGHT 600
// Desired aspect ratio. 640x480 = 4/3. Comment out to use whatever the desktop is set.
#define DESIRED_ASPECT ((float)(DESIRED_WINDOW_WIDTH)/(float)(DESIRED_WINDOW_HEIGHT))

//#define FULLSCREEN_BY_DEFAULT

//#define RESIZABLE_WINDOW

#define USE_PERFCOUNTERS

int tex_enemy, tex_wall, tex_hut, tex_circle, tex_halp;
int gKeyState = 0;
int gLastTick = 0;
int gScreenWidth = 0;
int gScreenHeight = 0;

int gamestate = 0;

SDL_AudioSpec *gAudioSpec = NULL;

struct Sample
{
    short * data;
    int length;
    SDL_AudioSpec spec;
};

Sample sound[6], au, go;

struct Channel
{
    Sample *audio;
    float pos;
    float inc;
};

#define MAX_CHANNELS 16

Channel channel[MAX_CHANNELS];

void playsound(Sample &sound)
{
    int i = 0;
    while (i < MAX_CHANNELS && channel[i].audio != NULL) i++;
    if (i >= MAX_CHANNELS)
    {
        i = gVisualRand.genrand_int31() % MAX_CHANNELS;
    }
    
    channel[i].pos = 0;
    channel[i].inc = 11025 / 44100.0f;
    channel[i].audio = &sound;
}


ACFont fn;
void initvideo(int argc);

void handle_key(int keysym, int down)
{
    switch(keysym)
    {
    case SDLK_ESCAPE:
        if (!down)
        {
            if (gamestate == 0)
                exit(0);
            gamestate = 0;
        }
        break;
    case SDLK_UP:
    case SDLK_w:
        if (down) gKeyState |= KEY_UP; else gKeyState &= ~KEY_UP;
        break;
    case SDLK_DOWN:
    case SDLK_s:
        if (down) gKeyState |= KEY_DOWN; else gKeyState &= ~KEY_DOWN;
        break;
    case SDLK_LEFT:
    case SDLK_a:
        if (down) gKeyState |= KEY_LEFT; else gKeyState &= ~KEY_LEFT;
        break;
    case SDLK_RIGHT:
    case SDLK_d:
        if (down) gKeyState |= KEY_RIGHT; else gKeyState &= ~KEY_RIGHT;
        break;
    case SDLK_RCTRL:
    case SDLK_LCTRL:
        if (down) gKeyState |= KEY_FIRE;
        break;
    }
}

void process_events()
{
    SDL_Event event;

    while (SDL_PollEvent(&event)) 
    {
        switch (event.type) 
        {
        case SDL_KEYDOWN:
            handle_key(event.key.keysym.sym, 1);
			// If a key is pressed, report it to the widgets
			gUIState.keyentered = event.key.keysym.sym;
			gUIState.keymod = event.key.keysym.mod;
			// if key is ASCII, accept it as character input
			if ((event.key.keysym.unicode & 0xFF80) == 0)
				gUIState.keychar = event.key.keysym.unicode & 0x7f;				
            break;
        case SDL_KEYUP:
            handle_key(event.key.keysym.sym, 0);
            break;
        case SDL_MOUSEMOTION:
			// update mouse position
			gUIState.mousex = (event.motion.x * (float)DESIRED_WINDOW_WIDTH) / (float)gScreenWidth;
			gUIState.mousey = (event.motion.y * (float)DESIRED_WINDOW_HEIGHT) / (float)gScreenHeight;
            break;
		case SDL_MOUSEBUTTONDOWN:
			// update button down state if left-clicking
			if (event.button.button == 1)
            {
				gUIState.mousedown = 1;
			    gUIState.mousedownx = (event.motion.x * (float)DESIRED_WINDOW_WIDTH) / (float)gScreenWidth;
			    gUIState.mousedowny = (event.motion.y * (float)DESIRED_WINDOW_HEIGHT) / (float)gScreenHeight;
            }
            if (event.button.button == 4)
            {
                gUIState.scroll = +1;
            }
            if (event.button.button == 5)
            {
                gUIState.scroll = -1;
            }
			break;
		case SDL_MOUSEBUTTONUP:
			// update button down state if left-clicking
			if (event.button.button == 1)
				gUIState.mousedown = 0;
			break;
        case SDL_QUIT:
            SDL_Quit();
            exit(0);
            break;
        case SDL_VIDEORESIZE:
            gScreenWidth = event.resize.w;
            gScreenHeight = event.resize.h;
            initvideo(0);
            break;

        }
    }
}

struct bloodparticle
{
    float x;
    float y;
    float xi;
    float yi;
    int live;
};

#define MAX_BLOOD 256
bloodparticle blood[MAX_BLOOD];

void spawn_blood(float x, float y)
{
    int idx = gVisualRand.genrand_int31() % MAX_BLOOD;

    blood[idx].x = x;
    blood[idx].y = y;
    blood[idx].live = gVisualRand.genrand_int31() % 100 + 100;
    float xi = gVisualRand.genrand_real1() - 0.5f;
    float yi = gVisualRand.genrand_real1() - 0.5f;
    float l = sqrt(xi*xi+yi*yi);
    float d = gVisualRand.genrand_real1() * 2;
    blood[idx].xi = (xi / l) * d;
    blood[idx].yi = (yi / l) * d;
}

class enemies 
{
public:
    float x, y;
    float xi, yi;
    int live;
};

#define MAX_ENEMY 256
enemies enemy[MAX_ENEMY] = {0};

class walls
{
public:
    float x0, y0, x1, y1;
    float xi, yi;
    int splats;
    int live;
};

#define MAX_WALLS 64
walls wall[MAX_WALLS] = {0};

int inputstate = 0;
int input_x0, input_y0;
int wave = 0;
int nextspawn = 100;
int score = 0;
int health = 109;

void reset()
{
    int i;
    for (i = 0; i < MAX_WALLS; i++)
        wall[i].live = 0;
    for (i = 0; i < MAX_ENEMY; i++)
        enemy[i].live = 0;

    inputstate = 0;
    wave = 0;
    nextspawn = 100;
    score = 0;
    health = 109;
}

void spawn_wall(float x0, float y0, float x1, float y1)
{
    int idx = 0;
    while (idx < MAX_WALLS && wall[idx].live) idx++;

    if (idx == MAX_WALLS)
        return;

    wall[idx].x0 = x0;
    wall[idx].y0 = y0;
    wall[idx].x1 = x1;
    wall[idx].y1 = y1;
    wall[idx].splats = 0;

    float xd = x0-x1;
    float yd = y0-y1;
    float len = sqrt(xd*xd+yd*yd);
    if (len < 8) return;
    int d = len / 20;
    if (d < 1) d = 1;
    health -= d;

    xd /= len;
    yd /= len;

    // are we moving in the right direction?

    if (-yd*((x0+x1)/2-400) + xd*((y0+y1)/2-300) < 0)
    {
        yd = -yd;
        xd = -xd;
        float t;
        t = wall[idx].x0;
        wall[idx].x0 = wall[idx].x1;
        wall[idx].x1 = t;
        t = wall[idx].y0;
        wall[idx].y0 = wall[idx].y1;
        wall[idx].y1 = t;
    }

    wall[idx].xi = -yd * 0.5;
    wall[idx].yi = xd * 0.5;
    wall[idx].live = 1;
}

void spawn_enemy(float x, float y, float spd)
{
    int idx = gPhysicsRand.genrand_int31() % MAX_ENEMY;
    if (enemy[idx].live)
        return;
    enemy[idx].x = x;
    enemy[idx].y = y;

    int destx = 375 + (gPhysicsRand.genrand_int31() % 50);
    int desty = 275 + (gPhysicsRand.genrand_int31() % 50);

    float len = sqrt((destx-x)*(destx-x) + (desty-y)*(desty-y));
    enemy[idx].xi = ((destx-x) / len) * spd * 0.1;
    enemy[idx].yi = ((desty-y) / len) * spd * 0.1;    
    enemy[idx].live = 1;
}

void physics_tick(int tick)
{
    physics_popups();

    int i, j, k;

    for (i = 0; i < MAX_BLOOD; i++)
    {
        if (blood[i].live)
        {
            blood[i].live--;
            blood[i].x += blood[i].xi;
            blood[i].y += blood[i].yi;
            blood[i].xi *= 0.9;
            blood[i].yi *= 0.9;
        }
    }

    for (i = 0; i < MAX_ENEMY; i++)
    {
        if (enemy[i].live)
        {
            enemy[i].x += enemy[i].xi;
            enemy[i].y += enemy[i].yi;

            if (sqrt((enemy[i].x-400) * (enemy[i].x-400) + ((enemy[i].y-300) * (enemy[i].y-300))) < 100)
            {
                // hit target
                health -= 10;
                playsound(au);
                
                if (health < 0) 
                {
                    health = 0;
                    gamestate = 0;
                }
                enemy[i].live = 0;
            }
             
            for (j = 0; j < MAX_WALLS; j++)
            {
                if (wall[j].live)
                {
                    if (rect_line_collide(enemy[i].x-16,enemy[i].y-16,enemy[i].x+16,enemy[i].y+16,wall[j].x0,wall[j].y0,wall[j].x1,wall[j].y1))
                    {
                        enemy[i].live = 0;
                        wall[j].splats++;
                        score += wall[j].splats;
                        playsound(sound[gVisualRand.genrand_int31()%6]);
                        char temp[64];
                        sprintf(temp,"+%d",wall[j].splats);
                        spawn_popup(temp,enemy[i].x,enemy[i].y);
                        for (k = 0; k < 17; k++)
                            spawn_blood(enemy[i].x,enemy[i].y);
                    }
                }
            }
        }
    }

    for (i = 0; i < MAX_WALLS; i++)
    {
        if (wall[i].live)
        {
            wall[i].x0 += wall[i].xi;
            wall[i].y0 += wall[i].yi;
            wall[i].x1 += wall[i].xi;
            wall[i].y1 += wall[i].yi;
            if (
                (wall[i].x0 > 800 && wall[i].x1 > 800) ||
                (wall[i].x0 < 0 && wall[i].x1 < 0) ||
                (wall[i].y0 > 600 && wall[i].y1 > 600) ||
                (wall[i].y0 < 0 && wall[i].y1 < 0))
            {
                // out of the screen
                wall[i].live = 0;
            }
        }
    }

    nextspawn--;
    if (nextspawn <= 0)
    {
        nextspawn = 1000;
        wave++;
        
        health += 10;
        if (health > 109) health = 109;

        for (i = 0; i < (wave + 10)/2; i++)
        {
            float x = gPhysicsRand.genrand_real1() - 0.5f;
            float y = gPhysicsRand.genrand_real1() - 0.5f;
            float s = gPhysicsRand.genrand_real1();
            float l = sqrt(x*x+y*y);
            x /= l;
            y /= l;
            spawn_enemy(x*800+400,y*600+300,s*7+1);
        }
    }
}

void draw_circle(int x, int y, int w, int c)
{
    glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
    drawtexturedrect(tex_circle,x-w/2,y-w/2,w,w,c);
}

void draw_wall(float x0, float y0, float x1, float y1, int splats, float alpha=1)
{
    glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
    glBindTexture(GL_TEXTURE_2D, tex_wall);
    glEnable(GL_TEXTURE_2D);
    glColor4f(1,1,1,alpha);

    float xd = x0-x1;
    float yd = y0-y1;
    float len = sqrt(xd*xd+yd*yd);

    xd /= len;
    yd /= len;

    xd *= 8;
    yd *= -8;

    float tick = SDL_GetTicks() / 1000.0f;

    float rx = 1.0f / (splats * 0.2f + 1);

    glBegin(GL_QUADS);
    glColor4f(0.5,0.5*rx,0.5*rx,alpha);
        glTexCoord2f(0+tick,0);
        glVertex3f(x0+yd,y0+xd,0);
        glTexCoord2f(0+tick,1*len/16);
        glVertex3f(x1+yd,y1+xd,0);
    glColor4f(1,1*rx,1*rx,alpha);
        glTexCoord2f(0.5+tick,1*len/16);
        glVertex3f(x1,y1,0);
        glTexCoord2f(0.5+tick,0);
        glVertex3f(x0,y0,0);

        glTexCoord2f(0.5+tick,0);
        glVertex3f(x0,y0,0);
        glTexCoord2f(0.5+tick,1*len/16);
        glVertex3f(x1,y1,0);
    glColor4f(0.5,0.5*rx,0.5*rx,alpha);
        glTexCoord2f(1+tick,1*len/16);
        glVertex3f(x1-yd,y1-xd,0);
        glTexCoord2f(1+tick,0);
        glVertex3f(x0-yd,y0-xd,0);
    glEnd();

    glDisable(GL_TEXTURE_2D);
}

int validwall(float x0, float y0, float x1, float y1)
{
    float xd = x0-x1;
    float yd = y0-y1;
    float len = sqrt(xd*xd+yd*yd);

    if (health <= (len / 20))
        return 0;

    // end point 1
    float l1 = sqrt((x0-400)*(x0-400)+(y0-300)*(y0-300));
    if (l1 < 100) return 0;
    if (l1 > 150) return 0;
    
    // end point 2
    float l2 = sqrt((x1-400)*(x1-400)+(y1-300)*(y1-300));      
    if (l2 < 100) return 0;
    if (l2 > 150) return 0;

    // midpoint
    float l3 = sqrt(((x1+x0)/2-400)*((x1+x0)/2-400)+((y1+y0)/2-300)*((y1+y0)/2-300));
    if (l3 < 100) return 0;
    if (l3 > 150) return 0;

    // that's enough checking.. and we'll give the player a bit slack this way too

    return 1;
}

static void draw_screen_ingame()
{
    int tick = SDL_GetTicks();

    int i, j, k;    

    if (tick - gLastTick > TICK_TIMEWARP) 
        gLastTick = tick;

    if (gLastTick >= tick)
    {
        SDL_Delay(1);
        return;
    }

    while (gLastTick < tick)
    {
        physics_tick(gLastTick);

        gLastTick += 1000 / PHYSICS_FPS;
    }

    if (gUIState.mousedown && inputstate == 0)
    {
        input_x0 = gUIState.mousedownx;
        input_y0 = gUIState.mousedowny;
        inputstate = 1;
    }
    if (inputstate == 1 && !gUIState.mousedown)
    {
        if (validwall(input_x0, input_y0, gUIState.mousex, gUIState.mousey))
        {
            spawn_wall(input_x0, input_y0, gUIState.mousex, gUIState.mousey);            
            playsound(go);
        }
        inputstate = 0;
    }


    ////////////////////////////////////
    // Rendering
    ////////////////////////////////////

    glClearColor(0.1,0.1,0.2,1.0);
    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

    ////////////////////////////////////
    // testcode
    ////////////////////////////////////

    char temp[1024];
    sprintf(temp, "SCORE: %d\nWAVE: %d", score, wave);
    fn.drawstring(temp,10,10,-1,24);
    
    draw_circle(400,300,300,0xff004f00);
    draw_circle(400,300,200,0xff002f00);
    /*
    drawsprite(tex_hut,400-40,300+20,32,32,1);
    drawsprite(tex_hut,400+10,300-40,32,32,1);
    drawsprite(tex_hut,400-70,300-20,32,32,1);
    */

    int d = 0;
    if (inputstate == 1)
    {
        float xd = input_x0-gUIState.mousex;
        float yd = input_y0-gUIState.mousey;
        float len = sqrt(xd*xd+yd*yd);
        d = len / 20;
        if (d < 1) d = 1;
    }

    d = health - d;

    j = health / 10;
    k = d / 10;
    for (i = 0; i < 5; i++)
    {
        if (j > i)
        {
            if (k > i)
                drawsprite(tex_hut,400-(31*5)/2+38*i,300-32,32,32,1);
            else
                drawsprite(tex_hut,400-(31*5)/2+38*i,300-32,24,24,1);
        }
        if (j > i + 5)
        {
            if (k > i + 5)
                drawsprite(tex_hut,400-(31*5)/2+38*i,300,32,32,1);
            else
                drawsprite(tex_hut,400-(31*5)/2+38*i,300,24,24,1);
        }
    }

    j = health % 10;
    k = d % 10;
    for (i = 0; i < j; i++)
    {
        if (i < k)
            drawsprite(tex_hut,400-(32*5)/2+18*i,300+32,16,16,1);   
        else
            drawsprite(tex_hut,400-(32*5)/2+18*i,300+32,12,12,1);   
    }
    
    for (i = 0; i < MAX_ENEMY; i++)
    {
        if (enemy[i].live)
        {
            drawsprite(tex_enemy,enemy[i].x,enemy[i].y,32,32,1);
        }
    }

    for (i = 0; i < MAX_BLOOD; i++)
    {
        if (blood[i].live)
        {
            drawrect(blood[i].x-1,blood[i].y-1,3,3,0xff0000|(blood[i].live << 24));
        }
    }

    for (i = 0; i < MAX_WALLS; i++)
    {
        if (wall[i].live)
        {
            draw_wall(wall[i].x0, wall[i].y0, wall[i].x1, wall[i].y1, wall[i].splats);
        }
    }

    if (inputstate == 1 && validwall(input_x0,input_y0,gUIState.mousex,gUIState.mousey))
        draw_wall(input_x0,input_y0,gUIState.mousex,gUIState.mousey,0,0.5);

    for (j = 0; j < MAX_POPUPS; j++)
    {
        if (popup[j].live)
        {
            fn.drawstring(popup[j].text,popup[j].x,popup[j].y,0x7fffffff);
        }
    }

    SDL_Delay(10);
	SDL_GL_SwapBuffers();
}


static void draw_screen_menu()
{
    int tick = SDL_GetTicks();

    glClearColor(0.1,0.1,0.2,1.0);
    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

    fn.drawstring(TITLE,320,50,-1,24);
    draw_wall(250, 90,250+300,90,0,1);
    draw_wall(250+300, 30,250,30,0,1);
    fn.drawstring("By Jari Komppa 2009\nfor the Ludum Dare 48h\ncontest #14:\n\"Advancing Wall of Doom\"",320,124);
    fn.drawstring("http://iki.fi/sol",5,600-32);

    char temp[256];
    sprintf(temp,"Last score:%d",score);
    fn.drawstring(temp,320,230);

    imgui_prepare();

    if (imgui_button(GEN_ID,fn,"Start game",200,300,400,48,0xff00003f,0xff1f1faf,0xff3f3faf,0xffffffff))
    {
        reset();
        gamestate = 1;
    }
    if (imgui_button(GEN_ID,fn,"Start brutal game",200,350,400,48,0xff00003f,0xff1f1faf,0xff3f3faf,0xffffffff))
    {
        reset();
        wave = 100;
        gamestate = 1;
    }

    if (imgui_button(GEN_ID,fn,"How to play",200,400,400,48,0xff00003f,0xff1f1faf,0xff3f3faf,0xffffffff))
    {
        gamestate = 2;
    }

    if (imgui_button(GEN_ID,fn,"Quit game",200,500,400,48,0xff00003f,0xff1f1faf,0xff3f3faf,0xffffffff))
    {
        exit(0);
    }

    imgui_finish();

    SDL_Delay(10);
	SDL_GL_SwapBuffers();
}


static void draw_screen_halp()
{
    int tick = SDL_GetTicks();

    glClearColor(0.1,0.1,0.2,1.0);
    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

    drawtexturedrect(tex_halp,0,0,800,600,0xffffffff);
    
    if (gUIState.mousedown)
    {
        gUIState.hotitem = 0;        
    }
    else
    {
        if (gUIState.hotitem == 0)
            gamestate = 0;
    }

    SDL_Delay(10);
	SDL_GL_SwapBuffers();
}



void draw_screen()
{
    switch (gamestate)
    {
    case 0:
        draw_screen_menu();
        break;
    case 1:
        draw_screen_ingame();
        break;
    case 2:
        draw_screen_halp();
        break;
    }
}

void initvideo(int argc)
{
    const SDL_VideoInfo *info = NULL;
    int bpp = 0;
    int flags = 0;

    info = SDL_GetVideoInfo();

    if (!info) 
    {
        fprintf(stderr, "Video query failed: %s\n", SDL_GetError());
        SDL_Quit();
        exit(0);
    }

#ifdef _DEBUG
    int fsflag = 0;
#else
#ifdef FULLSCREEN_BY_DEFAULT
    int fsflag = 1;
#else
    int fsflag = 0;
#endif
#endif

    if (argc > 1) fsflag = !fsflag;

    if (fsflag) 
    {
        gScreenWidth = info->current_w;
        gScreenHeight = info->current_h;
        bpp = info->vfmt->BitsPerPixel;
        flags = SDL_OPENGL | SDL_FULLSCREEN;
    }
    else
    {
        if (argc == 0)
        {
            // window was resized
        }
        else
        {
            gScreenWidth = DESIRED_WINDOW_WIDTH;
            gScreenHeight = DESIRED_WINDOW_HEIGHT;
        }
        bpp = info->vfmt->BitsPerPixel;
        flags = SDL_OPENGL;
#ifdef RESIZABLE_WINDOW
        flags |= SDL_RESIZABLE;
#endif
    }

    SDL_GL_SetAttribute(SDL_GL_RED_SIZE, 5);
    SDL_GL_SetAttribute(SDL_GL_GREEN_SIZE, 5);
    SDL_GL_SetAttribute(SDL_GL_BLUE_SIZE, 5);
    SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE, 16);
    SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);

    if (SDL_SetVideoMode(gScreenWidth, gScreenHeight, bpp, flags) == 0) 
    {
        fprintf( stderr, "Video mode set failed: %s\n", SDL_GetError());
        SDL_Quit();
        exit(0);
    }
   
#ifdef DESIRED_ASPECT
    float aspect = DESIRED_ASPECT;
    if (((float)gScreenWidth / gScreenHeight) > aspect)
    {
        float realx = gScreenHeight * aspect;
        float extrax = gScreenWidth - realx;

        glViewport( extrax / 2, 0, realx, gScreenHeight );
    }
    else
    {
        float realy = gScreenWidth / aspect;
        float extray = gScreenHeight - realy;

        glViewport( 0, extray / 2, gScreenWidth, realy );
    }
#else
    glViewport( 0, 0, gScreenWidth, gScreenHeight );
#endif

    glMatrixMode( GL_PROJECTION );
    glLoadIdentity( );
#ifdef DESIRED_ASPECT
	gluOrtho2D(0,DESIRED_WINDOW_WIDTH,DESIRED_WINDOW_HEIGHT,0);
#else
    gluOrtho2D(0,gScreenWidth,gScreenHeight,0);
#endif

    reload_textures();    

    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA,GL_ONE);
}

void audiomixer(void *userdata, Uint8 *stream, int len)
{
    int n = len / 2;
    short * buf = (short*)stream;
    memset(stream,0,len);
    int i;
    for (i = 0; i < n; i++)
    {
        float f = 0;
        int j;
        for (j = 0; j < MAX_CHANNELS; j++)
        {
            if (channel[j].audio)
            {
                int ipos = (int)floor(channel[j].pos);
                f += channel[j].audio->data[ipos];
                channel[j].pos += channel[j].inc;
                if (channel[j].audio->length/2 <= channel[j].pos)
                    channel[j].audio = NULL;
            }
        }
        buf[i] = (short)(f / 2.0f); // sampledata
    }
}


int main (int argc, char** args)
{
    gVisualRand.init_genrand(0xc0cac01a);
    reset();
    int i;
    for (i = 0; i < MAX_CHANNELS; i++)
        channel[i].audio = NULL;

    if (SDL_Init(SDL_INIT_VIDEO | SDL_INIT_AUDIO | SDL_INIT_NOPARACHUTE) < 0) 
    {
        fprintf(stderr, "Video initialization failed: %s\n", SDL_GetError());
        SDL_Quit();
        exit(0);
    }

    SDL_AudioSpec *as = new SDL_AudioSpec;
    as->freq = 44100;
    as->format = AUDIO_S16;
    as->channels = 2;
    as->samples = 4096;
    as->callback = audiomixer;
    as->userdata = NULL;
    if (SDL_OpenAudio(as, NULL) < 0)
    {
        fprintf(stderr, "Unable to init SDL audio: %s\n", SDL_GetError());
        exit(1);
    }
    gAudioSpec = as;

    initvideo(argc);

    tex_enemy = load_texture("data/enemy.png");
    tex_wall = load_texture("data/awod.png",0);
    tex_hut = load_texture("data/hut.png");
    tex_circle = load_texture("data/circle.png");
    tex_halp = load_texture("data/halp.png");

    SDL_LoadWAV("data/k1.wav",&sound[0].spec, (Uint8**)&sound[0].data, (Uint32*)&sound[0].length);
    SDL_LoadWAV("data/k2.wav",&sound[1].spec, (Uint8**)&sound[1].data, (Uint32*)&sound[1].length);
    SDL_LoadWAV("data/k3.wav",&sound[2].spec, (Uint8**)&sound[2].data, (Uint32*)&sound[2].length);
    SDL_LoadWAV("data/k4.wav",&sound[3].spec, (Uint8**)&sound[3].data, (Uint32*)&sound[3].length);
    SDL_LoadWAV("data/k5.wav",&sound[4].spec, (Uint8**)&sound[4].data, (Uint32*)&sound[4].length);
    SDL_LoadWAV("data/k6.wav",&sound[5].spec, (Uint8**)&sound[5].data, (Uint32*)&sound[5].length);
    SDL_LoadWAV("data/au.wav",&au.spec, (Uint8**)&au.data, (Uint32*)&au.length);
    SDL_LoadWAV("data/go.wav",&go.spec, (Uint8**)&go.data, (Uint32*)&go.length);

	// For imgui - Enable keyboard repeat to make sliders more tolerable
	SDL_EnableKeyRepeat(SDL_DEFAULT_REPEAT_DELAY, SDL_DEFAULT_REPEAT_INTERVAL);
	// For imgui - Enable keyboard UNICODE processing for the text field.
	SDL_EnableUNICODE(1);

    // load a font..
    fn.load("data/vera16.fnt");
    
    // set window title
    SDL_WM_SetCaption(TITLE " - http://iki.fi/sol/", NULL);

    SDL_PauseAudio(0);

    // hide cursor
//    SDL_ShowCursor(0);
    
    while (1) 
    {
        process_events();
        draw_screen();
    }

    return 0;
}
